@@ -26,12 +26,14 @@ typedef enum {
 	PREPARING_TO_WRITE_CD,
 	WRITING_CD,
 	FIXATING_CD,
+	BLANKING_CD,
 } ActionType;
 
 static char *actions [] = {
 	N_("Preparing to write CD"),
 	N_("Writing CD"),
 	N_("Fixating CD"),
+	N_("Erasing CD"),
 	NULL
 };
 
@@ -57,6 +59,7 @@ enum {
 	ANIMATION_CHANGED,
 	CANCEL_FUNC_CHANGED,
 	INSERT_CD_REQUEST,
+	INSERT_RWCD_REQUEST,
 	LAST_SIGNAL
 };
 
@@ -176,6 +179,10 @@ cdrecord_stdout_read (GIOChannel   *source,
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
 				       cancel_cdrecord, TRUE);
+		} else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")) {
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[ACTION_CHANGED], 0,
+				       BLANKING_CD);
 		}
 
 		g_free (line);
@@ -220,6 +227,11 @@ cdrecord_stderr_read (GIOChannel   *source,
 				       cd_recorder_table_signals[INSERT_CD_REQUEST], 0,
 				       FALSE);
 			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+		} else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_RWCD_REQUEST], 0,
+				       FALSE);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
 		} else if (strstr (line, "Data may not fit on current disk") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The files selected did not fit on the CD"));
 		} else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
@@ -248,6 +260,7 @@ cd_recorder_write_iso (CDRecorder *cdrecorder,
 		       const char *filename,
 		       int speed,
 		       gboolean eject,
+		       gboolean blank,
 		       gboolean dummy_write)
 {
 	const char *argv[20]; /* Shouldn't need more than 20 arguments */
@@ -272,6 +285,9 @@ cd_recorder_write_iso (CDRecorder *cdrecorder,
 	if (eject) {
 		argv[i++] = "-eject";
 	}
+	if (blank) {
+		argv[i++] = "blank=fast";
+	}
 	argv[i++] = "-v";
 	argv[i++] = "-data";
 	argv[i++] = filename;
@@ -474,5 +490,15 @@ cd_recorder_class_init (CDRecorderClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__BOOLEAN,
 				G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+	cd_recorder_table_signals[INSERT_RWCD_REQUEST] =
+		g_signal_new ("insert-rwcd-request",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				G_STRUCT_OFFSET (CDRecorderClass,
+					insert_cd_rw_request),
+				NULL, NULL,
+				g_cclosure_marshal_VOID__BOOLEAN,
+				G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+
 }
 
