@@ -223,6 +223,16 @@ insert_cd_response (GtkDialog *dialog, gint response_id, gpointer data)
 	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
 }
 
+static void
+insert_cdrw_response (GtkDialog *dialog, gint response_id, gpointer data)
+{
+	CDRecorder *cdrecorder = (CDRecorder *) data;
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	cd_recorder_insert_cd_retry (cdrecorder, FALSE);
+}
+
 static void
 show_reload_dialog (CDRecorder *cdrecorder)
 {
@@ -255,6 +265,22 @@ show_insert_dialog (CDRecorder *cdrecorder)
 	gtk_widget_show (insert_cd_dialog);
 }
 
+static void
+show_insert_rw_dialog (CDRecorder *cdrecorder)
+{
+	GtkWidget *insert_cdrw_dialog;
+
+	insert_cdrw_dialog = gtk_message_dialog_new (cd_progress_get_window (),
+						   GTK_DIALOG_DESTROY_WITH_PARENT,
+						   GTK_MESSAGE_INFO,
+						   GTK_BUTTONS_OK,
+						   _("Please insert a blank CD-RW in the CD writer."));
+	gtk_window_set_title (GTK_WINDOW (insert_cdrw_dialog),
+			      _("Insert blank CD-RW"));
+	g_signal_connect (insert_cdrw_dialog, "response", (GCallback)insert_cdrw_response, cdrecorder);
+	gtk_widget_show (insert_cdrw_dialog);
+}
+
 static void
 insert_cd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
 		      gpointer data)
@@ -266,6 +292,17 @@ insert_cd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
 	}
 }
 
+static void
+insert_rwcd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
+		      gpointer data)
+{
+	if (is_reload) {
+		show_reload_dialog (cdrecorder);
+	} else {
+		show_insert_rw_dialog (cdrecorder);
+	}
+}
+
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
@@ -384,7 +421,7 @@ setup_close_button (void)
 }
 
 static int
-burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, int speed, gboolean dummy)
+burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy)
 {
 	CDRecorder *cdrecorder;
 	GtkWidget *filesel;
@@ -459,7 +496,11 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-cd-request",
 				  G_CALLBACK (insert_cd_request_cb),
 				  NULL);
-		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, dummy);
+		g_signal_connect (G_OBJECT (cdrecorder),
+				  "insert-rwcd-request",
+				  G_CALLBACK (insert_rwcd_request_cb),
+				  NULL);
+		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, blank_cd, dummy);
 		if (source_iso == NULL) {
 			unlink (filename);
 		}
@@ -505,6 +546,7 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *menu;
 	GtkWidget *item;
 	GtkWidget *eject_check;
+	GtkWidget *blank_check;
 	GList *l, *children;
 	char *name;
 	int i;
@@ -545,6 +587,10 @@ refresh_dialog (GladeXML *xml)
 	/* Disable eject for burn-to-iso */
 	eject_check = glade_xml_get_widget (xml, "eject_check");
 	gtk_widget_set_sensitive (eject_check, rec->type != CDDRIVE_TYPE_FILE);
+
+	/* Disable blank for burn-to-iso */
+	blank_check = glade_xml_get_widget (xml, "blank_check");
+	gtk_widget_set_sensitive (blank_check, rec->type != CDDRIVE_TYPE_FILE);
 	
 }
 
@@ -557,7 +603,7 @@ target_changed (GtkOptionMenu *option_menu, gpointer data)
 }
 
 static void
-init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name)
+init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean disable_reuse)
 {
 	GList *l;
 	CDDrive *rec;
@@ -566,6 +612,7 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *item;
 	GtkWidget *label;
 	GtkWidget *entry;
+	GtkWidget *reuse;
 	struct tm *tm;
 	int size_mb;
 	char *str;
@@ -612,6 +659,11 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
 
+	reuse = glade_xml_get_widget (xml, "reuse_check");
+	g_assert (GTK_IS_WIDGET (reuse));
+	gtk_widget_set_sensitive (reuse, !disable_reuse);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reuse), TRUE);
+
 	refresh_dialog (xml);
 }
 
@@ -724,11 +776,11 @@ int
 main (int argc, char *argv[])
 {
 	CDDrive *rec;
-	GtkWidget *dialog, *entry, *eject_check, *reuse_check, *progress_window;
+	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
 	int res, speed;
-	gboolean eject, reuse, dummy;
+	gboolean eject, blank, reuse, dummy;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
 	char *source_iso, *disc_name = NULL;
@@ -803,7 +855,7 @@ main (int argc, char *argv[])
 	xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	dialog = glade_xml_get_widget (xml, "cdr_dialog");
 
-	init_dialog (xml, dialog, size, disc_name);
+	init_dialog (xml, dialog, size, disc_name, source_iso != NULL);
 
 	if (disc_name != NULL) g_free (disc_name);
 
@@ -834,6 +886,9 @@ main (int argc, char *argv[])
 		eject_check = glade_xml_get_widget (xml, "eject_check");
 		eject = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (eject_check));
 
+		blank_check = glade_xml_get_widget (xml, "blank_check");
+		blank = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blank_check));
+
 		dummy = FALSE;
 
 #ifdef ENABLE_DUMMY_WRITE
@@ -844,7 +899,7 @@ main (int argc, char *argv[])
 		item = gtk_menu_get_active (GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu))));
 		speed = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "speed"));
 
-		res = burn_cd (rec, source_iso, label, eject, speed, dummy);
+		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
